@@ -14,18 +14,22 @@
 #' m1 = cbind(c(0, 0, 1, 0), c(0, 1, 1, 0))
 #' m2 = cbind(c(0, 1, 1, 0), c(0, 0, 1, 0))
 #' pol = st_sfc(st_polygon(list(m1)), st_polygon(list(m2)))
-#' library(sf)
 #' set.seed(1985)
 #' d = data.frame(matrix(runif(15), ncol = 3))
 #' p = st_as_sf(x = d, coords = 1:2)
 #' plot(pol)
 #' plot(p, add = TRUE)
-#' (p_ag = aggregate(p, pol, mean))
-#' plot(p_ag) # geometry same as pol
+#' (p_ag1 = aggregate(p, pol, mean))
+#' plot(p_ag1) # geometry same as pol
 #' # works when x overlaps multiple objects in 'by':
 #' p_buff = st_buffer(p, 0.2)
 #' plot(p_buff, add = TRUE)
-#' aggregate(p_buff, pol, mean) # increased mean of second
+#' (p_ag2 = aggregate(p_buff, pol, mean)) # increased mean of second
+#' # with non-matching features
+#' m3 = cbind(c(0, 0, -0.1, 0), c(0, 0.1, 0.1, 0))
+#' pol = st_sfc(st_polygon(list(m3)), st_polygon(list(m1)), st_polygon(list(m2)))
+#' (p_ag3 = aggregate(p, pol, mean))
+#' plot(p_ag3)
 #' @export
 aggregate.sf = function(x, by, FUN, ..., do_union = TRUE, simplify = TRUE,
 		join = st_intersects) {
@@ -38,8 +42,15 @@ aggregate.sf = function(x, by, FUN, ..., do_union = TRUE, simplify = TRUE,
 		# dispatch to stats::aggregate:
 		a = aggregate(x[unlist(i), , drop = FALSE],
 			list(rep(seq_len(nrow(by)), lengths(i))), FUN, ...)
+		nrow_diff = nrow(by) - nrow(a)
+		if(nrow_diff > 0) {
+			a_na = a[rep(NA, nrow(by)),] # 'top-up' missing rows
+			a_na[a$Group.1,] = a
+			a = a_na
+		}
 		a$Group.1 = NULL
-		st_set_geometry(a, st_geometry(by)[lengths(i) > 0])
+		row.names(a) = row.names(by)
+		st_set_geometry(a, st_geometry(by))
 	} else {
 		crs = st_crs(x)
 		lst = lapply(split(st_geometry(x), by), function(y) do.call(c, y))
