@@ -4,10 +4,11 @@
 #' @importFrom methods as slotNames new slot
 #' @importFrom grid convertUnit current.viewport linesGrob pathGrob pointsGrob polylineGrob unit viewport nullGrob
 #' @import graphics
+#' @importFrom grDevices rgb
 #' @importFrom Rcpp evalCpp
 #' @importFrom DBI dbConnect dbDisconnect dbWriteTable dbGetQuery dbSendQuery dbReadTable dbExecute
 #' @importFrom units make_unit set_units
-#' @importFrom grDevices rgb
+#' @importFrom classInt classIntervals
 #' @useDynLib sf
 NULL
 
@@ -63,6 +64,7 @@ setOldClass("sfg")
 #' Provide the external dependencies versions of the libraries linked to sf
 #' @export
 sf_extSoftVersion = function() {
-	structure(c(CPL_geos_version(), CPL_gdal_version(), CPL_proj_version(), CPL_lwgeom_version()),
-		names = c("GEOS", "GDAL", "proj.4", "lwgeom"))
+	structure(c(CPL_geos_version(), CPL_gdal_version(), CPL_proj_version(), CPL_lwgeom_version(),
+		ifelse(CPL_gdal_with_geos(), "true", "false")),
+		names = c("GEOS", "GDAL", "proj.4", "lwgeom", "GDAL_with_GEOS"))
 }
