@@ -12,8 +12,11 @@ getClassDim = function(x, d, dim = "XYZ", type) {
 	else stop(paste(d, "is an illegal number of columns for a", type))
 }
 
+is_numeric_matrix = function(x)
+	stopifnot(is.numeric(x) && is.matrix(x))
+
 Mtrx = function(x, dim = "XYZ", type) {
-	stopifnot(is.matrix(x) && is.numeric(x))
+	is_numeric_matrix(x)
 	structure(x, class = getClassDim(x, ncol(x), dim, type))
 }
 
@@ -24,10 +27,11 @@ MtrxSet = function(x, dim = "XYZ", type, needClosed = FALSE) {
 		nc = unique(vapply(x, ncol, 0L))
 		if (length(nc) != 1)
 			stop("matrices having unequal number of columns")
+		lapply(x, is_numeric_matrix)
 		NotClosed = function(y) any(head(y, 1) != tail(y, 1))
 		if (needClosed && any(vapply(x, NotClosed, TRUE)))
 			stop("polygons not (all) closed")
-		structure(x, class = getClassDim(x, ncol(x[[1]]), dim, type))
+		structure(x, class = getClassDim(x, nc, dim, type))
 	} else
 		structure(x, class = getClassDim(x, nchar(dim), dim, type))
 }
@@ -39,10 +43,11 @@ MtrxSetSet = function(x, dim = "XYZ", type, needClosed = FALSE) {
 		nc = unique(unlist(lapply(x, function(y) vapply(y, ncol, 0L))))
 		if (length(nc) != 1)
 			stop("matrices having unequal number of columns")
+		lapply(x, function(y) lapply(y, is_numeric_matrix))
 		NotClosed = function(y) any(head(y, 1) != tail(y, 1))
 		if (needClosed && any(unlist(lapply(x, function(y) vapply(y, NotClosed, TRUE)))))
 			stop("polygons not (all) closed")
-		structure(x, class = getClassDim(x, ncol(x[[1]][[1]]), dim, type))
+		structure(x, class = getClassDim(x, nc, dim, type))
 	} else
 		structure(x, class = getClassDim(x, nchar(dim), dim, type))
 }
