@@ -7,6 +7,7 @@
 #' @param ... ignored
 #' @param partial logical; allow for partial projection, if not all points of a geometry can be projected (corresponds to setting environment variable \code{OGR_ENABLE_PARTIAL_REPROJECTION} to \code{TRUE})
 #' @param check logical; perform a sanity check on resulting polygons?
+#' @param use_gdal logical; if \code{FALSE}, arguments \code{check} and \code{partial} get ignored, and projection is directly carried out by \code{lwgeom_transform} which ignores the GDAL api. This allows proj.4 parameters such as \code{+over} to act (https://github.com/r-spatial/sf/issues/511) and certain projections such as \code{wintri} which do not have an inverse, to work (https://github.com/r-spatial/sf/issues/509). This requires that sf is built by linking to \code{liblwgeom}.
 #' @details Transforms coordinates of object to new projection. Features that cannot be transformed are returned as empty geometries.
 #' @examples
 #' p1 = st_point(c(7,52))
@@ -56,11 +57,18 @@ sanity_check = function(x) {
 #' @export
 #' @examples
 #' st_transform(st_sf(a=2:1, geom=sfc), "+init=epsg:3857")
-st_transform.sfc = function(x, crs, ..., partial = TRUE, check = FALSE) {
+st_transform.sfc = function(x, crs, ..., partial = TRUE, check = FALSE, use_gdal = TRUE) {
 	if (is.na(st_crs(x)))
 		stop("sfc object should have crs set")
 	if (missing(crs))
 		stop("argument crs cannot be missing")
+
+	if (! use_gdal) {
+		if (inherits(crs, "crs"))
+			crs = crs$proj4string
+		return(st_sfc(CPL_lwgeom_transform(x, c(st_crs(x)$proj4string, crs))))
+	}
+
 	crs = make_crs(crs)
 
 	if (grepl("+proj=geocent", crs$proj4string) && length(x) && Dimension(x[[1]]) == "XY") # add z:
@@ -142,3 +150,9 @@ st_wrap_dateline = function(x, options = "WRAPDATELINE=YES", quiet = TRUE) {
 	stopifnot(is.logical(quiet) && length(quiet) == 1)
 	st_sfc(CPL_wrap_dateline(x, options, quiet), crs = st_crs(x))
 }
+
+st_to_s2 = function(x) {
+	# geocentric, spherical:
+	st_transform(x, 
+		st_crs("+proj=geocent +a=1 +b=1 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +wktext +no_defs"))
+}
