@@ -22,8 +22,8 @@
 //
 static void __err_handler(CPLErr eErrClass, int err_no, const char *msg)
 {
-    switch ( eErrClass )
-    {
+	switch ( eErrClass )
+	{
         case 0:
             break; // #nocov
         case 1:
@@ -107,6 +107,19 @@ Rcpp::List CPL_crs_parameters(std::string p4s) {
 	return out;
 }
 
+// [[Rcpp::export]]
+Rcpp::LogicalVector CPL_crs_equivalent(std::string crs1, std::string crs2) {
+	Rcpp::LogicalVector out(1);
+	OGRSpatialReference *srs1 = new OGRSpatialReference;
+	handle_error(srs1->importFromProj4(crs1.c_str()));
+	OGRSpatialReference *srs2 = new OGRSpatialReference;
+	handle_error(srs2->importFromProj4(crs2.c_str()));
+	out(0) = (bool) srs1->IsSame(srs2);
+	delete srs1;
+	delete srs2;
+	return out;
+}
+
 std::vector<OGRGeometry *> ogr_from_sfc(Rcpp::List sfc, OGRSpatialReference **sref) {
 	double precision = sfc.attr("precision");
 	Rcpp::List wkblst = CPL_write_wkb(sfc, false, native_endian(), get_dim_sfc(sfc, NULL), precision);
@@ -402,3 +415,24 @@ Rcpp::List CPL_sfc_from_wkt(Rcpp::CharacterVector wkt) {
 	}
 	return sfc_from_ogr(g, true);
 }
+
+// [[Rcpp::export]]
+Rcpp::LogicalVector CPL_gdal_with_geos() {
+
+	bool withGEOS;
+	CPLPushErrorHandler(CPLQuietErrorHandler);
+	OGRGeometry *poGeometry1, *poGeometry2;
+	char* pszWKT = (char *) "POINT (10 20)";
+	OGRGeometryFactory::createFromWkt( &pszWKT, NULL, &poGeometry1 );
+	pszWKT = (char *) "POINT (30 20)";
+	OGRGeometryFactory::createFromWkt( &pszWKT, NULL, &poGeometry2 );
+	withGEOS = 1;
+	if (poGeometry1->Union(poGeometry2) == NULL) 
+		withGEOS = false; // #nocov
+	else
+		withGEOS = true;
+	OGRGeometryFactory::destroyGeometry(poGeometry1);
+	OGRGeometryFactory::destroyGeometry(poGeometry2);
+	CPLPopErrorHandler();
+	return Rcpp::LogicalVector::create(withGEOS);
+}
