@@ -9,6 +9,8 @@ extern "C" {
 #include <liblwgeom.h>
 }
 
+using namespace Rcpp; // for _ to work
+
 // [[Rcpp::export]]
 Rcpp::CharacterVector CPL_lwgeom_version(bool b = false) {
 	return lwgeom_version();
@@ -82,6 +84,36 @@ Rcpp::CharacterVector CPL_geohash(Rcpp::List sfc, int prec) {
 	return chr;
 }
 
+// [[Rcpp::export]]
+Rcpp::List CPL_lwgeom_transform(Rcpp::List sfc, Rcpp::CharacterVector p4s) {
+	if (p4s.size() != 2)
+		Rcpp::stop("st_lwgeom_transform: p4s needs to be a length 2 character vector\n"); // #nocov
+	std::vector<LWGEOM *> lwgeom_v = lwgeom_from_sfc(sfc);
+	projPJ src = lwproj_from_string(p4s[0]);
+	if (src == NULL)
+		Rcpp::stop("st_lwgeom_transform: wrong source proj4string\n"); // #nocov
+	projPJ target = lwproj_from_string(p4s[1]);
+	if (target == NULL)
+		Rcpp::stop("st_lwgeom_transform: wrong target proj4string\n"); // #nocov
+	for (int i = 0; i < lwgeom_v.size(); i++) {
+		// in-place transformation w/o GDAL:
+		if (lwgeom_transform(lwgeom_v[i], src, target) != LW_SUCCESS) {
+			Rcpp::Rcout << "Failed on geometry " << i + 1 << std::endl; // #nocov
+			Rcpp::stop("st_lwgeom_transform failed\n"); // #nocov
+		}
+	}
+	pj_free(src);
+	pj_free(target);
+	Rcpp::List ret = sfc_from_lwgeom(lwgeom_v); // frees lwgeom_v
+	Rcpp::List crs = Rcpp::List::create(
+		_["epsg"] = NA_INTEGER, 
+		_["proj4string"] = CharacterVector::create(p4s[1]));
+	crs.attr("class") = "crs";
+	ret.attr("crs") = crs;
+	ret.attr("class") = "sfc";
+	return ret;
+}
+
 #else
 // #nocov start
 
@@ -104,5 +136,10 @@ Rcpp::CharacterVector CPL_geohash(Rcpp::List sfc, int prec) {
 	return NA_STRING;
 }
 
+Rcpp::List CPL_lwgeom_transform(Rcpp::List sfc, Rcpp::CharacterVector p4s) {
+	Rcpp::stop("st_transform with use_gdal = FALSE requires compilation against liblwgeom\n");
+	return sfc;
+}
+
 // #nocov end
 #endif
